 #include <iostream>
#include <string>
#include <fstream>
#include <iomanip>

#include <TMath.h>
#include <TTree.h>
#include <TFile.h>
#include <TH1.h>
#include <TH2.h>
#include <TBenchmark.h>     // ROOT head file for benchmarking applications

using namespace std;

int main(int argc, char *argv[])
{
  //
  // time statistic --- start
  TBenchmark* gBenchmark = new TBenchmark();
  //start time of this program
  gBenchmark->Start("main");

  // process commandline
  if(argc<2) {
    cout<<"Usage: checkdata --help to get help"<<endl;
    return -1;
  }
  string datafilename="unknown";
  if (argc>=2){
    for (int i=1;i<(argc);i++){
      if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")){
        cout<<"Usage: ----------->"<<endl;
        cout<<argv[0]<<" -f datafilename "<<endl;
        return 0;
      }
      if (!strcmp(argv[i],"-f"))   {datafilename=argv[++i];}
    }
  }
  char tmpchr[200];
  //-----------------
  sprintf(tmpchr,"%s",datafilename.c_str());
  FILE* inputfile=fopen(tmpchr,"r");
  if(!inputfile) {cout<<"Can not open "<<tmpchr<<endl;return -1;};  
  cout<<"Process Data File:"<<tmpchr<<" "<<endl;
  sprintf(tmpchr,"%s",datafilename.c_str());

  //==================================================================================
  //--read in INL tables
  char inltablename[200];
  sprintf(inltablename,"tdig-ser48");
  double inlcor[3][8][1024];
  for(int i=0;i<1024;i++){for(int ich=0;ich<8;ich++){for(int itdc=0;itdc<3;itdc++)inlcor[itdc][ich][i]=0;}}

  for(int itdc=0;itdc<3;itdc++){
    for(int ich=0;ich<8;ich++){
      // read in inltable file 
      sprintf(tmpchr,"../result/%s/inl/%s.tdc%d.ch%d.inl",inltablename,inltablename,itdc,ich);
      ifstream inlinput(tmpchr);
      if(!inlinput){cout<<"ERROR!! Can not find "<<tmpchr<<endl;return -1;}
      cout<<"INL table:"<<tmpchr<<endl;

      float tbin,cor;
      int index=0;
      while(inlinput>>tbin>>cor){
        int ibin = int(tbin-0.5);
        inlcor[itdc][ich][ibin]=cor;
        index++;
      }  
    }   
  }
  // end read INL tables
  float ltime[2],ttime[2];
  TTree* hitstree = new TTree("hitstree","hits");
  hitstree->Branch("ltime",ltime,"ltime[2]/F");
  hitstree->Branch("ttime",ttime,"ttime[2]/F");

  TH1F* ToT1h[2];
  ToT1h[0]=new TH1F("PMT1_ToT","PMT1_ToT",400,0,50);
  ToT1h[1]=new TH1F("PMT2_ToT","PMT2_ToT",400,0,50);

  TH1F* firstToT1h[2];
  firstToT1h[0]=new TH1F("PMT1_ToT1","PMT1_ToT1",400,0,50);
  firstToT1h[1]=new TH1F("PMT2_ToT1","PMT2_ToT1",400,0,50);

  TH1F* deltatime1h=new TH1F("deltatime1h","deltatime1h",280,-5,9);
  TH2F* ToT1vsdeltatime2h=new TH2F("ToT1vsdeltatime2h","ToT1vsdeltatime2h",400,0,50,400,-5,5);  
  TH2F* ToT2vsdeltatime2h=new TH2F("ToT2vsdeltatime2h","ToT2vsdeltatime2h",400,0,50,400,-5,5);  
  //======================================================================= 
  unsigned int itime,tempv;
  float ftime=0;
  int numret;
  int tdig,edgeid,tdcid,tdcchan;
  int atdig[50],aedgeid[50],atdcid[50],atdcchan[50];
  float altime[50],attime[50];

  int hitevents=0,wrongedgeevents=0,events=0,pairevents=0;

  float lasttime=0;
  int nhits=0;
  while ((numret=fscanf(inputfile,"%x",&tempv))==1) {

    if(tempv>>16==0xe000) {cout<<" ----------- Find a Seperator: "<<hex<<tempv<<endl;continue;}

    edgeid =int( (tempv & 0xf0000000)>>28 );
    if((edgeid != 4) && (edgeid != 5)){cout<<"ERROR!!----------------Wrong edge="<<edgeid<<endl;wrongedgeevents++;continue;} 

    tdcid = (tempv & 0x0F000000)>>24;  tdcid = tdcid % 8;
    tdcchan = (tempv&0x00E00000)>>21;
    itime=((tempv&0x7ffff)<<2)+((tempv>>19)&0x00000003);
    tdig = tdcid/4;
    int bin=itime&0x000003ff;
    ftime= (itime+inlcor[tdcid][tdcchan][bin])*25./1024.;       

    if(tdcid==0 && tdcchan==0) ltime[0]=ftime;
    if(tdcid==2 && tdcchan==0) ltime[1]=ftime;
    if(tdcid==1 && tdcchan==7) ttime[0]=ftime;
    if(tdcid==1 && tdcchan==5) ttime[1]=ftime;
    hitstree->Fill();

    if(TMath::Abs(lasttime-ftime)<80.) {  
      nhits++;lasttime=ftime;
      atdig[nhits]=tdig;aedgeid[nhits]=edgeid;atdcid[nhits]=tdcid;atdcchan[nhits]=tdcchan;
      altime[nhits]=ftime;

    } else {
      //for(int i=0;i<nhits+1;i++)cout<<"time===="<<altime[i]<<endl;
      // make ToT
      int pmtl1=0,pmtl2=0,pmtt1=0,pmtt2=0;
      float pmt1letime[50],pmt1tetime[50],pmt2letime[50],pmt2tetime[50];
      float firstle1=1.e10,firstle2=1.e10,firstte1=1.e10,firstte2=1.e10;
      for(int i=0;i<50;i++){pmt1letime[i]=0;pmt1tetime[i]=0;pmt2letime[i]=0;pmt2tetime[i]=0;}
      for(int i=0;i<nhits+1;i++){
        if(atdcid[i]==0 && atdcchan[i]==0&&aedgeid[i]==4) {pmt1letime[pmtl1]=altime[i];pmtl1++;}
        if(atdcid[i]==2 && atdcchan[i]==0&&aedgeid[i]==4) {pmt2letime[pmtl2]=altime[i];pmtl2++;}
        if(atdcid[i]==1 && atdcchan[i]==7&&aedgeid[i]==5) {pmt1tetime[pmtt1]=altime[i];pmtt1++;}
        if(atdcid[i]==1 && atdcchan[i]==5&&aedgeid[i]==5) {pmt2tetime[pmtt2]=altime[i];pmtt2++;}
      }
      int nloop=pmtl1;if(pmtl1>pmtt1) nloop=pmtt1;
      for(int i1=0;i1<nloop;i1++){
        if(firstle1>pmt1letime[i1])firstle1=pmt1letime[i1];
        for(int i2=0;i2<nloop;i2++){
          if(firstte1>pmt1tetime[i1])firstte1=pmt1tetime[i1];
          float ToT = pmt1tetime[i2]-pmt1letime[i1];
          ToT1h[0]->Fill(ToT);
	}
      }
      if(nloop>0)firstToT1h[0]->Fill(firstte1-firstle1);

      nloop=pmtl2;if(pmtl2>pmtt2) nloop=pmtt2;
      for(int i1=0;i1<nloop;i1++){
        if(firstle2>pmt2letime[i1])firstle2=pmt2letime[i1];
        for(int i2=0;i2<nloop;i2++){
          if(firstte2>pmt2tetime[i1])firstte2=pmt2tetime[i1];
          float ToT = pmt2tetime[i2]-pmt2letime[i1];
          ToT1h[1]->Fill(ToT);
	}
      }

      if(nloop>0)firstToT1h[1]->Fill(firstte2-firstle2);
      //--------------------------------------      
      // find coincidence events
      if(nhits+1>3 && pmtl1>0 && pmtl2>0 && (TMath::Abs(firstle1-firstle2-1)<5) && pmtt1>0 && pmtt2>0) {
	deltatime1h->Fill(firstle2-firstle1);
        ToT1vsdeltatime2h->Fill(firstte1-firstle1,firstle2-firstle1);
        ToT2vsdeltatime2h->Fill(firstte2-firstle2,firstle2-firstle1);
        pairevents++;
      }
      // --------------------------------------

      events++;
      nhits=0;lasttime=ftime;
      atdig[nhits]=tdig;aedgeid[nhits]=edgeid;atdcid[nhits]=tdcid;atdcchan[nhits]=tdcchan;
      altime[nhits]=ftime;
    }  // end else
    cout<<"events="<<events<<" value:0x"<<hex<<tempv<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<tdcid<<" tdcch="<<tdcchan<<" time="<<itime<<" ftime="<<ftime<<endl; 

    hitevents++;
  }
  fclose(inputfile);
  TFile* outfile=new TFile("cosmic_analysis.root","recreate");
  hitstree->Write();
  ToT1h[0]->Write();
  ToT1h[1]->Write();
  firstToT1h[0]->Write();
  firstToT1h[1]->Write();
  deltatime1h->Write();
  ToT1vsdeltatime2h->Write();
  ToT2vsdeltatime2h->Write();

  outfile->Close();

  cout<<"Processed total hits= "<<hitevents<<" realevents="<<events<<" coincidence events="<<pairevents<<" , with wrong edge events="<<wrongedgeevents<<endl;

  //=================================================================================
  cout<<" All done. "<<endl;
  // time statistics.
  gBenchmark->Stop("main");       // run time stop here
  gBenchmark->Show("main");       // print out the time information
  float realtime = gBenchmark->GetRealTime("main");        // real time elapsed
  float cputime  = gBenchmark->GetCpuTime("main");         // CPU time we use
  cout<<" real time = "<<realtime<<" seconds cput="<<cputime<<" seconds"<<endl;

  return 0;
}
