#include <iostream>
#include <string>
#include <fstream>
#include <iomanip>

#include <TFile.h>
#include <TH1.h>
#include <TH2.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TPaveLabel.h>
#include <TVirtualPad.h>
#include <TGaxis.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TLegend.h>
#include <TROOT.h>
#include <TLatex.h>
#include <TBenchmark.h>     // ROOT head file for benchmarking applications
#include <TApplication.h>

using namespace std;
void CreateDir(const string);

int main(int argc, char *argv[])
{

  // need this line to make "gif" files online....do not run code in batch mode.
  TApplication* theApp=new TApplication("app",0 ,0);
  //
  // time statistic --- start
  TBenchmark* gBenchmark = new TBenchmark();
  //start time of this program
  gBenchmark->Start("main");

  // process commandline
  if(argc<2) {
    cout<<"Usage: noise_analysis --help to get help"<<endl;
    return -1;
  }
  string datafilename="unknown";
  if (argc>1){
    for (int i=1;i<(argc);i++){
      if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")){
        cout<<"Usage: ----------->"<<endl;
        cout<<argv[0]<<" -f datafilename "<<endl;
        return 0;
      }
      if (!strcmp(argv[i],"-f"))   {datafilename=argv[++i];}
    }
  }
  cout<<"filename = "<<datafilename<<endl;

  string ComFileName=datafilename;
  int dotpos=ComFileName.find(".dat");  
  if(dotpos>0)ComFileName=ComFileName.erase(dotpos,ComFileName.length());  
  cout<<"noise_analysis: "<<ComFileName<<endl;

  char tmpchr[200];
  char filename[200];
  sprintf(filename,"%s",ComFileName.c_str());
  
  cout<<"filename="<<filename<<endl;

//=====================================================
  gROOT->SetStyle("Plain");

  gStyle->SetCanvasBorderMode(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetPadBorderMode(0);
  gStyle->SetDrawBorder(0);
  gStyle->SetCanvasBorderSize(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetStatBorderSize(1);
  gStyle->SetTitleBorderSize(0);

  // Say it in black and white!
  gStyle->SetLabelFont(62,"xyz");
  gStyle->SetTitleOffset(1.25,"Y");
  gStyle->SetTitleOffset(1.,"X");
  gStyle->SetTitleSize(0.06);
  gStyle->SetOptFile(1);

  // Set Line Widths
  gStyle->SetFrameLineWidth(2);
  gStyle->SetFuncWidth(2);

  // Set margins -- I like to shift the plot a little up and to the
  // right to make more room for axis labels
  gStyle->SetPadTopMargin(0.08);
  gStyle->SetPadBottomMargin(0.12);
  gStyle->SetPadLeftMargin(0.12);
  gStyle->SetPadRightMargin(0.08);
  //
  // Set Data/Stat/... and other options
  // Set tick marks and turn off grids
  gStyle->SetNdivisions(505,"xyz");
  gStyle->SetPadGridX(0);
  gStyle->SetPadGridY(0);
  // Adjust size and placement of axis labels
  gStyle->SetLabelSize(0.09,"x");
  gStyle->SetLabelSize(0.09,"y");
  gStyle->SetLabelOffset(0.005,"x");
  gStyle->SetLabelOffset(0.01,"y");
  // Set paper size for life in the US
  gStyle->SetPaperSize(TStyle::kUSLetter);

  gStyle->SetOptStat(0);
  gStyle->SetOptTitle(0);


  gStyle->SetPalette(1);
  //
  gROOT->ForceStyle();

  TH2F* noiserate2h[8];
  for(int iboard=0;iboard<4;iboard++){
    sprintf(tmpchr,"hits_board%d",iboard);
    float xx=iboard*4;
    noiserate2h[iboard]=new TH2F(tmpchr,tmpchr,3,xx-0.5,3+xx-0.5,8,-0.5,7.5);
  } 
  for(int iboard=4;iboard<8;iboard++){
    sprintf(tmpchr,"hits_board%d",iboard);
    float xx=(iboard-4)*4; 
    noiserate2h[iboard]=new TH2F(tmpchr,tmpchr,3,xx-0.5,3+xx-0.5,8,-0.5,7.5);
  } 
  TH1F* ToT1h=new TH1F("ToT","ToT",100,0,40);

  //-----------------
  sprintf(tmpchr,"%s.dat",filename);
  FILE* inputfile=fopen(tmpchr,"r");
  if(!inputfile) {cout<<"Can not open "<<tmpchr<<endl;return -1;};  
  cout<<"Process Data File:"<<tmpchr<<" "<<endl;

  int hitperchan[8][3][8]; for(int i=0;i<8;i++){for(int j=0;j<3;j++){for(int k=0;k<8;k++){hitperchan[i][j][k]=0;}}}
  unsigned int tempv,tempv1;
  int numret=0;
  int tdcid, tdcchan, tdig,edgeid, otdcid;
  int lsb;
  int lastword=0;
  int totalwords=0,goodwords=0;
  int bin=0;
  unsigned int itime;
  float time;
  int wrongedge=0;
  int printwords=0;
  TCanvas* c1=new TCanvas("c1","c1",0,0,1000,500);c1->SetFillColor(0);
  c1->Divide(8,1,0,0);

  int halftrayid=0;

  while ((numret=fscanf(inputfile,"%x",&tempv))==1) {
    fscanf(inputfile,"%x",&tempv);
   
    //cout<<"-----------------------------------------------------------------------------"<<endl;
    if(totalwords%10000==1&&goodwords>lastword){lastword=goodwords;cout<<" processed data words: "<<dec<<goodwords<<endl;}
    totalwords++;

    if(tempv>>16==0xe000) continue;

    //if((tempv & 0xFFF)>>20 ==0x405) continue;
    //if((tempv & 0xFFF)>>20 ==0x404) continue;
    
    edgeid =int( (tempv & 0xf0000000)>>28 );
    if((edgeid != 4) && (edgeid != 5) && (edgeid !=2)){if(wrongedge<50)cout<<"ERROR!!Wrong edge="<<edgeid<<" value=0x"<<hex<<tempv<<endl;wrongedge++;continue;} 
    //cout<<" edgeid="<<edgeid<<" value=0x"<<hex<<tempv<<endl;
    if(edgeid==2) {halftrayid=(tempv & 0x0F000000)>>24;continue;}
    //if(edgeid !=4) continue;
    halftrayid=0;
    //cout<<"halftrayid="<<halftrayid<<" value=0x"<<hex<<tempv<<endl;
    otdcid = (tempv & 0x0F000000)>>24;  
    tdcid = (tempv & 0x0F000000)>>24;  
    tdig = tdcid/4 +halftrayid*4;tdcid = tdcid % 4;
    if(tdig<0 || tdig >=8) {cout<<"ERROR!! TDIG ID ="<<tdig<<" check your configuration !!"<<endl;return -1;}
    tdcchan = (tempv&0x00e00000)>>21;
    if(tdcchan<0 || tdcchan >8){cout<<"ERROR!! TDCchan="<<tdcchan<<endl;return -1;}

    goodwords++;

    itime=((tempv&0x7ffff)<<2)+((tempv>>19)&0x00000003);
    //if(goodwords<3000)cout<<"value:0x"<<hex<<tempv<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<otdcid<<" tdcchan="<<tdcchan<<" itime="<<itime<<" goodwords="<<goodwords<<endl; 
    if(printwords<3000 && !(tdig==0 && otdcid==0 && tdcchan==2) ){
      cout<<"value:0x"<<hex<<tempv<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<otdcid<<" tdcchan="<<tdcchan<<" itime="<<itime<<" goodwords="<<goodwords<<endl; 
      printwords++;
    }
    lsb=((tempv&0x1ff)<<2)+((tempv>>19)&0x00000003);
    bin=itime&0x000003ff;
    time=itime ;
    hitperchan[tdig][tdcid][tdcchan]++;
    noiserate2h[tdig]->Fill(otdcid,tdcchan);

    int maxnhit=0;
    for(int i=0;i<8;i++){
      if(noiserate2h[i]->GetMaximum()>maxnhit) maxnhit=noiserate2h[i]->GetMaximum();
    }
/*
    for(int i=0;i<8;i++){
      c1->cd(i+1);
      //noiserate2h[i]->SetMaximum(maxnhit);
      noiserate2h[i]->SetMarkerSize(5);
      noiserate2h[i]->Draw("col text");
    }
   
    if(totalwords%200==1) c1->Update();
*/
 }
  fclose(inputfile);
  cout<<"Process:  ="<<dec<<totalwords<<" good words "<<goodwords<<" wrong edge words ="<<wrongedge<<endl;

    for(int i=0;i<8;i++){
      c1->cd(i+1);
      //noiserate2h[i]->SetMaximum(maxnhit);
      noiserate2h[i]->SetMarkerSize(5);
      noiserate2h[i]->Draw("col text");
    }
   

  // write histograms into root file.

  sprintf(tmpchr,"%s.noise.root",filename);
  cout<<"Write to root file :"<<tmpchr<<endl;
  TFile* fFile=new TFile(tmpchr,"recreate");
  for(int iboard=0;iboard<8;iboard++){noiserate2h[iboard]->Write();}
  fFile->Close();

  theApp->Run();  
        
  //=================================================================================
  cout<<" All done. "<<endl;
  // time statistics.
  gBenchmark->Stop("main");       // run time stop here
  gBenchmark->Show("main");       // print out the time information
  float realtime = gBenchmark->GetRealTime("main");        // real time elapsed
  float cputime  = gBenchmark->GetCpuTime("main");         // CPU time we use
  cout<<" real time = "<<realtime<<" seconds cput="<<cputime<<" seconds"<<endl;

  return 0;
}


//-------------------------------
void CreateDir(const string directoryname)
{
  string command;
  // if the directoryname does not exist, creat it
  if(gSystem->AccessPathName(directoryname.c_str())){
    command = "mkdir "+directoryname; 
    cout<<"mkdir :"<<directoryname<<endl;
    gSystem->Exec(command.c_str()); //execute shell command to cread directory
  }
}
