#define _FILE_OFFSET_BITS  64

#include <iostream>
#include <string>
#include <fstream>
#include <iomanip>
#include <TFile.h>
#include <TH1.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TPaveLabel.h>
#include <TVirtualPad.h>
#include <TGaxis.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TLegend.h>
#include <TROOT.h>
#include <TBenchmark.h>     // ROOT head file for benchmarking applications
#include <TApplication.h>
#include <TH2.h>
#include <TMath.h>

#include <TBenchmark.h>     // ROOT head file for benchmarking applications

using namespace std;
void CreateDir(const string);

int main(int argc, char *argv[])
{

  // need this line to make "gif" files online....do not run code in batch mode.
  //  TApplication* theApp=new TApplication("app",&argc ,argv);
  TApplication* theApp=new TApplication("app",0 ,0);

  // time statistic --- start
  TBenchmark* gBenchmark = new TBenchmark();
  //start time of this program
  gBenchmark->Start("main");

  // process commandline
  if(argc<2) {
    cout<<"Usage: corr_analysis --help to get help"<<endl;
    return -1;
  }
  string datafilename="unknown";
  string inltablename="unknown";
  string processmode="8";
  int totchan=8;
  int dotdc=0;
  if (argc>=2){
    for (int i=1;i<(argc);i++){
      if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")){
        cout<<"Usage: ----------->"<<endl;
        cout<<argv[0]<<" -f datafilename "<<endl;
        cout<<argv[0]<<" -c inltablename -- Apply INL correction by inlfilename"<<endl;
        cout<<argv[0]<<" -tdc give a tdc id"<<endl;
        cout<<argv[0]<<" -mode processmode, correct INL by 1 or 8 table"<<endl;
        return 0;
      }

      if (!strcmp(argv[i],"-f"))   {datafilename=argv[++i];}
      if (!strcmp(argv[i],"-c"))   {inltablename=argv[++i];}
      if (!strcmp(argv[i],"-mode")) {processmode=argv[++i];}
      if (!strcmp(argv[i],"-tdc"))   {dotdc=atoi(argv[++i]);}
    }
  }

  char tmpchr[200];
  char filename[200];
  string ComFileName=datafilename;
  int dotpos=ComFileName.find(".dat");  
  if(dotpos>0)ComFileName=ComFileName.erase(dotpos,ComFileName.length());  
  cout<<"corr_analysis: "<<ComFileName<<endl;
  sprintf(filename,"%s",ComFileName.c_str());

  if(inltablename=="unknown")inltablename=filename;

  //create some directory, if not exist
  sprintf(tmpchr,"../result/%s",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/inl",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/root",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/gif",filename);CreateDir(tmpchr);

  double inlcor[8][1024];
  for(int i=0;i<1024;i++){for(int ich=0;ich<8;ich++){inlcor[ich][i]=0;}}

  for(int ich=0;ich<8;ich++){
    // read in inltable file 
    sprintf(tmpchr,"../result/%s/inl/%s.tdc%d.inl",inltablename.c_str(),inltablename.c_str(),dotdc);
    if(processmode=="8")sprintf(tmpchr,"../result/%s/inl/%s.tdc%d.ch%d.inl",inltablename.c_str(),inltablename.c_str(),dotdc,ich);
    ifstream inlinput(tmpchr);
    if(!inlinput){cout<<"ERROR!! Can not find "<<tmpchr<<endl;return -1;}
    cout<<"INL table:"<<tmpchr<<endl;

    float tbin,cor;
    int index=0;
    while(inlinput>>tbin>>cor){
      int ibin = int(tbin-0.5);
      inlcor[ich][ibin]=cor;
      index++;
    }  
   
    if(index<1024) {cout<<"ERROR! Not Enough correction constant in INL table !"<<endl;return -1;}
    cout<<"  First 8 inlcor=";
    for(int i=0;i<8;i++)cout<<inlcor[ich][i]<<" ";
    cout<<endl;
    cout<<"  Last  8 inlcor=";
    for(int i=0;i<8;i++)cout<<inlcor[ich][1024+i-8]<<" ";
    cout<<endl;
  }

  char hisname[200];
  TH1F* deltatimeh[totchan][totchan];
  TH1F* deltatime0h[totchan][totchan];
  for(int i=0;i<totchan;i++){
    for(int j=0;j<totchan;j++){
      sprintf(hisname,"TDC%d_deltaT_ch%d_ch%d",dotdc,i,j);
      deltatimeh[i][j]=new TH1F(hisname,hisname,40000,0,20000);
      sprintf(hisname,"TDC%d_deltaT0_ch%d_ch%d",dotdc,i,j);
      deltatime0h[i][j]=new TH1F(hisname,hisname,40000,0,20000);
    }
  }

  sprintf(tmpchr,"mean_vs_sigma_tdc%d",dotdc);
  TH2F* meanvssigma2h=new TH2F(tmpchr,tmpchr,2000,0,20000,60,0.3,3.3);
  sprintf(tmpchr,"all_sigma_tdc%d",dotdc);
  TH1F* sigma1h=new TH1F(tmpchr,tmpchr,54,0.3,3.);
 
  sprintf(tmpchr,"../rawdata/%s.dat",filename);
  FILE* inputfile=fopen(tmpchr,"r");
  if(!inputfile) {cout<<"Can not open "<<tmpchr<<endl;return -1;};  
  cout<<"Process Data File:"<<tmpchr<<" "<<processmode<<" total chan="<<totchan<<endl;

  int numret=0;
  int tdcid, tdcchan, tdig;

  unsigned int tempv[totchan],value;
  int edgeid;
  int misswordevents=0;
  int goodevents=0;
  int channelnum[totchan];
  //int lsb[totchan];
  float time[totchan],ntime[totchan];
  int lastevents=0;
  while ((numret=fscanf(inputfile,"%x",&tempv[0]))==1) {
    //cout<<"-----------------------------------------------------------------------------"<<endl;
    if(goodevents%500000==1&&goodevents>lastevents) {lastevents=goodevents;cout<<" processed events "<<goodevents<<endl;}
    if(tempv[0]>>16==0xe000) continue;
    edgeid =int( (tempv[0] & 0xf0000000)>>28 );if((edgeid != 4) && (edgeid != 5)) {continue;} 

    tdcid = (tempv[0] & 0x0F000000)>>24;  
    tdig=tdcid/3; tdcid = tdcid % 3;
    if(tdcid !=dotdc) continue;

    for(int i=0;i<totchan;i++){channelnum[i]=-1;}

    tdcchan = (tempv[0]&0x00E00000)>>21;
    channelnum[tdcchan]=tdcchan;
    
    for(int i=1;i<totchan;i++){
      tempv[i]=0;
      numret=fscanf(inputfile,"%x",&value);if(numret!=1) continue;
      if(value>>16==0xe000) continue;
      edgeid =int( (value & 0xf0000000)>>28 );if((edgeid != 4) && (edgeid != 5)) {continue;} 
      tdcid = (value & 0x0F000000)>>24;  
      tdig=tdcid/3; tdcid = tdcid % 3;
      if(tdcid !=dotdc) continue;
      tempv[i]=value;
      tdcchan = (value&0x00E00000)>>21;
      channelnum[tdcchan]=tdcchan;
    }

    int totalfiredchan=0; int misschan[8]={-1,-1,-1,-1,-1,-1,-1,-1};
    for(int i=0;i<totchan;i++){if(channelnum[i]>-1)totalfiredchan++;if(channelnum[i]<0)misschan[i]=i;} 
   
    if(totalfiredchan !=totchan){
       misswordevents++;
       if(misswordevents<50)cout<<" Missed words --------total hit chan= "<<totalfiredchan<<endl;
       for(int i=0;i<totchan;i++){
         if(tempv[i]==0) continue;
         if(channelnum[i]<0) continue;
         tdcid = (tempv[i] & 0x0F000000)>>24;
         tdig=tdcid/3; tdcid = tdcid % 3;
         tdcchan = (tempv[i]&0x00E00000)>>21;
         if(misswordevents<50)cout<<"value:0x"<<hex<<tempv[i]<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<tdcid<<" tdcchan="<<tdcchan<<endl;
       }
       continue;
    }

    for(int i=0;i<totchan;i++){channelnum[i]=-1;} 
    goodevents++;

    for(int i=0;i<totchan;i++){
      tdcid = (tempv[i] & 0x0F000000)>>24;
      tdig=tdcid/3; tdcid = tdcid % 3;
      tdcchan = (tempv[i]&0x00E00000)>>21;
      unsigned int itime=((tempv[i]&0x7ffff)<<2)+((tempv[i]>>19)&0x00000003);
      if(goodevents<10)cout<<"value:0x"<<hex<<tempv[i]<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<tdcid<<" tdcchan="<<tdcchan<<" itime="<<itime<<endl; 
      //lsb[i]=((tempv[i]&0x1ff)<<2)+((tempv[i]>>19)&0x00000003);
      int bin=itime&0x000003ff;
      ntime[tdcchan]=itime;
      time[tdcchan]=itime+inlcor[tdcchan][bin];
      
    }
    for(int i=0;i<totchan;i++){
      for(int j=0;j<totchan;j++){
        float deltatime=TMath::Abs(time[i]-time[j]);
        if(deltatime>1.e6)  deltatime=2097152-deltatime;   
        float deltatime0=TMath::Abs(float(ntime[i])-float(ntime[j]));
        if(deltatime0>1.e6)  deltatime0=2097152-deltatime0;   
        //if(deltatime>20000) cout<<"event="<<goodevents<<" chan ="<<i<<" time="<<time[i]<<" chan="<<j<<" time="<<time[j]<<endl;
        deltatimeh[i][j]->Fill(deltatime);
        deltatime0h[i][j]->Fill(deltatime0);
      }
    }
    for(int i=0;i<totchan;i++){tempv[i]=0;}
  }

  fclose(inputfile);
  cout<<"Good events = "<<goodevents<<" missed word events = "<<misswordevents<<endl;
  //make mean-vs-sigma plot.
  for(int i=0;i<8;i++){
    for(int j=0;j<8;j++){
      if(j<=i) continue;
      float mean=deltatimeh[i][j]->GetMean();
      float sigma=deltatimeh[i][j]->GetRMS(); 
      meanvssigma2h->Fill(mean,sigma);
      sigma1h->Fill(sigma);

      meanvssigma2h->SetMarkerColor(1+dotdc);
      meanvssigma2h->SetMarkerStyle(20+dotdc);
      meanvssigma2h->SetMarkerSize(1);

      sigma1h->SetLineColor(1+dotdc);
      sigma1h->SetLineWidth(2);
       
    }
  }

  // write everything to root file
  sprintf(tmpchr,"../result/%s",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/root",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/gif",filename); CreateDir(tmpchr);

  sprintf(tmpchr,"../result/%s/root/%s.tdc%d.corr1.root",filename,filename,dotdc);
  if(processmode=="8")sprintf(tmpchr,"../result/%s/root/%s.tdc%d.corr8.root",filename,filename,dotdc);
  cout<<"Write to root file :"<<tmpchr<<endl;
  TFile* fFile=new TFile(tmpchr,"recreate");

  for(int i=0;i<totchan;i++){for(int j=0;j<totchan;j++){deltatime0h[i][j]->Write();}}
  for(int i=0;i<totchan;i++){for(int j=0;j<totchan;j++){deltatimeh[i][j]->Write();}}
  meanvssigma2h->Write(); sigma1h->Write();

  fFile->Close();
  //=======================================
  // make gif file----
  //-- The following is a copy from plotdeltatime.C
  //
  gROOT->SetStyle("Plain");
  // Turn off all borders
  gStyle->SetCanvasBorderMode(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetPadBorderMode(0);
  gStyle->SetDrawBorder(0);
  gStyle->SetCanvasBorderSize(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetTitleBorderSize(1);

  gStyle->SetOptTitle(0);

  gStyle->SetNdivisions(505,"xyz");

  gStyle->SetLabelSize(0.08,"xyz");
  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetLabelSize(0.08,"x");
  gStyle->SetLabelOffset(0.01,"x");
  gStyle->SetLabelOffset(0.01,"y");
  gStyle->SetTickLength(0.09);
  gStyle->SetTitleColor(2,"xyz");
  gStyle->SetTitleOffset(1.3,"y");
  gStyle->SetTitleX(0.); gStyle->SetTitleY(1.);
  gStyle->SetTitleW(0.98); gStyle->SetTitleH(0.08);

  gStyle->SetStatX(0.91); gStyle->SetStatY(0.91);
  gStyle->SetStatW(0.28); gStyle->SetStatH(0.22);
  gStyle->SetStatFormat("12.6g");
  gStyle->SetOptFit(11);
  gStyle->SetOptStat("nemruo");
  gStyle->SetPadGridX(0);
  gStyle->SetPadGridY(0);
  //----
  TCanvas* c1=new TCanvas("c1","c1",0,0,750,650);
  c1->SetFillColor(0);c1->Divide(2,2,0.01,0.01);
  //TGaxis* A=new TGaxis(); A->SetMaxDigits(2);
  gROOT->ForceStyle();

  int ipage=0;
  int ifig=0;
  int lastpage=0;
  for(int i=0;i<8;i++){
    for(int j=0;j<8;j++){
      if(j<=i) continue;
      float mean=deltatimeh[i][j]->GetMean();
      deltatimeh[i][j]->SetAxisRange(mean-11,mean+22,"X");
      deltatimeh[i][j]->SetLineColor(4);
      deltatimeh[i][j]->SetLineWidth(2);
      c1->cd(ifig%4+1);
      deltatimeh[i][j]->Draw();
      ifig++;
      ipage=ifig/4;
      if(lastpage !=ipage) {
        sprintf(tmpchr,"../result/%s/gif/%s-tdc%d-page%d-deltatime.gif",filename,filename,dotdc,ipage);
        c1->Print(tmpchr);
        lastpage=ipage;
      }
    }
  }
  // end plot

  //----------------------------------------------------
  cout<<" All done. "<<endl;
  // time statistics.
  gBenchmark->Stop("main");       // run time stop here
  gBenchmark->Show("main");       // print out the time information
  float realtime = gBenchmark->GetRealTime("main");        // real time elapsed
  float cputime  = gBenchmark->GetCpuTime("main");         // CPU time we use
  cout<<" real time = "<<realtime<<" seconds cput="<<cputime<<" seconds"<<endl;

  return 0;
}

//-------------------------------
void CreateDir(const string directoryname)
{
  string command;
  // if the directoryname does not exist, creat it
  if(gSystem->AccessPathName(directoryname.c_str())){
    command = "mkdir "+directoryname; 
    cout<<"mkdir :"<<directoryname<<endl;
    gSystem->Exec(command.c_str()); //execute shell command to cread directory
  }
}
