
#define _FILE_OFFSET_BITS  64

#include <iostream>
#include <string>
#include <fstream>
#include <iomanip>

#include <TFile.h>
#include <TH1.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TPaveLabel.h>
#include <TVirtualPad.h>
#include <TGaxis.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TLegend.h>
#include <TROOT.h>
#include <TLatex.h>
#include <TBenchmark.h>     // ROOT head file for benchmarking applications
#include <TApplication.h>

using namespace std;
void CreateDir(const string);

int main(int argc, char *argv[])
{

  // need this line to make "gif" files online....do not run code in batch mode.
  //TApplication* theApp=new TApplication("app",&argc ,argv);
  TApplication* theApp=new TApplication("app",0 ,0);
  //
  // time statistic --- start
  TBenchmark* gBenchmark = new TBenchmark();
  //start time of this program
  gBenchmark->Start("main");

  string processmode="8";
  // process commandline
  if(argc<2) {
    cout<<"Usage: inl_analysis --help to get help"<<endl;
    return -1;
  }
  string datafilename="unknown";
  if (argc>=2){
    for (int i=1;i<(argc);i++){
      if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")){
        cout<<"Usage: ----------->"<<endl;
        cout<<argv[0]<<" -f datafilename "<<endl;
        cout<<argv[0]<<" -mode 1, or 8 "<<endl;
        return 0;
      }
      if (!strcmp(argv[i],"-f"))   {datafilename=argv[++i];}
      if (!strcmp(argv[i],"-mode"))   {processmode=argv[++i];}
    }
  }

  string ComFileName=datafilename;
  int dotpos=ComFileName.find(".dat");  
  if(dotpos>0)ComFileName=ComFileName.erase(dotpos,ComFileName.length());  
  cout<<"inl_analysis: "<<ComFileName<<endl;

  char tmpchr[200];
  char filename[200];
  sprintf(filename,"%s",ComFileName.c_str());

  float inlcor[1024];for(int i=0;i<1024;i++){inlcor[i]=0;}
  //
  TH1F* lsb2048h[3][8];
  TH1F* dnl2048h[3][8];
  TH1F* dnl1024h[3][8];
  TH1F* inl2048h[3][8];
  TH1F* inl1024h[3][8];

  for(int itdc=0;itdc<3;itdc++){
    for(int ich=0;ich<8;ich++){
      sprintf(tmpchr,"TDC%dch%d_lsb2048h",itdc,ich);
      lsb2048h[itdc][ich] = new TH1F(tmpchr,tmpchr,2048,0.5,2048.5);
      sprintf(tmpchr,"TDC%dch%d_dnl2048h",itdc,ich);
      dnl2048h[itdc][ich] = new TH1F(tmpchr,tmpchr,2048,0.5,2048.5);
      sprintf(tmpchr,"TDC%dch%d_dnl1024h",itdc,ich);
      dnl1024h[itdc][ich] = new TH1F(tmpchr,tmpchr,1024,0.5,1024.5);
      sprintf(tmpchr,"TDC%dch%d_inl2048h",itdc,ich);
      inl2048h[itdc][ich] = new TH1F(tmpchr,tmpchr,2048,0.5,2048.5);
      sprintf(tmpchr,"TDC%dch%d_inl1024h",itdc,ich);
      inl1024h[itdc][ich] = new TH1F(tmpchr,tmpchr,1024,0.5,1024.5);
    }
  }
  /*
  // test
  TH1F* tdc0ch0time1h=new TH1F("tdc0ch0time","tdc0ch0time",2097152,0,2097152);
  TH1F* tdc1ch0time1h=new TH1F("tdc1ch0time","tdc1ch0time",2097152,0,2097152);
  TH1F* tdc2ch0time1h=new TH1F("tdc2ch0time","tdc2ch0time",2097152,0,2097152);
  */
  //-----------------
  sprintf(tmpchr,"../rawdata/%s.dat",filename);
  FILE* inputfile=fopen(tmpchr,"r");

  cout<<" here = "<<inputfile<<endl;
  if(!inputfile) {cout<<"Can not open "<<tmpchr<<endl;return -1;};  
  cout<<"Process Data File:"<<tmpchr<<" "<<endl;

  //create some directory:
  sprintf(tmpchr,"../result/%s",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/inl",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/root",filename);CreateDir(tmpchr);
  sprintf(tmpchr,"../result/%s/gif",filename);CreateDir(tmpchr);


  unsigned int tempv;
  int numret=0;
  int tdcid, tdcchan, tdig,edgeid;
  int lsb;
  int lastevents=0;
  int goodevents=0;
  int bin=0;
  unsigned int itime;
  float time;
  int wrongedge=0;
  while ((numret=fscanf(inputfile,"%x",&tempv))==1) {
    //cout<<"-----------------------------------------------------------------------------"<<endl;
    if(goodevents%1000000==1&&goodevents>lastevents){lastevents=goodevents;cout<<" processed events "<<goodevents<<endl;}
    if(tempv>>16==0xe000) continue;

    tdcid = (tempv & 0x0F000000)>>24;  
    tdig = tdcid/3;tdcid = tdcid % 3;
    if(tdcid >=3) {cout<<"TDC ID =="<<tdcid<<" check your configuration !!"<<endl;return -1;}
    tdcchan = (tempv&0x00E00000)>>21;
    if(tdcchan<0 || tdcchan >8){cout<<"TDCid="<<tdcid<<" TDCchan="<<tdcchan<<" ERROR!!"<<endl;return -1;}

    edgeid =int( (tempv & 0xf0000000)>>28 );
    if((edgeid != 4) && (edgeid != 5)){if(wrongedge<50)cout<<"ERROR!!Wrong edge="<<edgeid<<endl;wrongedge++;continue;} 
    //tdcchan = (tempv&0x00E00000)>>21;
    //channelnum=tdcchan;
    goodevents++;

    //if(goodevents<60000000) continue;

    itime=((tempv&0x7ffff)<<2)+((tempv>>19)&0x00000003);
    if(goodevents<100)cout<<"value:0x"<<hex<<tempv<<dec<<" TDIG="<<tdig<<" edge="<<edgeid<<" tdcid="<<tdcid<<" tdcchan="<<tdcchan<<" itime="<<itime<<endl; 
    lsb=((tempv&0x1ff)<<2)+((tempv>>19)&0x00000003);
    bin=itime&0x000003ff;
    time=itime ;
    // fill histogram:
    if(processmode=="8")lsb2048h[tdcid][tdcchan]->Fill(lsb);
    if(processmode=="1"){for(int ich=0;ich<8;ich++){lsb2048h[tdcid][ich]->Fill(lsb);}}
    /*
    if(goodevents<100*24) {
      if(tdcid==0 && tdcchan==0) tdc0ch0time1h->Fill(itime);
      if(tdcid==1 && tdcchan==0) tdc1ch0time1h->Fill(itime);
      if(tdcid==2 && tdcchan==0) tdc2ch0time1h->Fill(itime);
    }
    */
    //
  }
  fclose(inputfile);
  cout<<"Good events = "<<goodevents<<" wrong edge words ="<<wrongedge<<endl;

  // now make INL table.....
  float lsbdata[2048];      
  float dnl2048[2048];
  float inl2048[2048];

  for(int itdc=0;itdc<3;itdc++){
    for(int ich=0;ich<8;ich++){
      //----------------------------------------------------------------
      float hihit=-1;float lowhit=99999999;
      float hidnl=-20000;float lowdnl=20000;
      float TotHit=0.;float TotBin=0;
      for(int ibin=0;ibin<lsb2048h[itdc][ich]->GetNbinsX();ibin++){
        Stat_t count=lsb2048h[itdc][ich]->GetBinContent(ibin);
        lsbdata[ibin]=count;
        TotHit=TotHit+count;
        TotBin++;
      }

      float interdnl=0;
      double expectedhit=double(TotHit)/double(TotBin);
      for(int ibin=0;ibin<TotBin;ibin++){
        dnl2048[ibin]=(lsbdata[ibin] - expectedhit )/expectedhit;
        if(dnl2048[ibin]>hidnl) hidnl=dnl2048[ibin];
        if(dnl2048[ibin]<lowdnl)lowdnl=dnl2048[ibin];
        if(lsbdata[ibin]>hihit) hihit=lsbdata[ibin];
        if(lsbdata[ibin]<lowhit)lowhit=lsbdata[ibin];
        interdnl=interdnl+dnl2048[ibin];
        inl2048[ibin]=interdnl;
        dnl2048h[itdc][ich]->SetBinContent(ibin,dnl2048[ibin]);
        inl2048h[itdc][ich]->SetBinContent(ibin,interdnl);
      }

      // 1024 inltable ..................
      TotHit=0.;TotBin=0.;
      for(int ibin=0;ibin<1024;ibin++){
        TotHit=TotHit+lsbdata[ibin];
        TotBin++;
      }

      expectedhit=double(TotHit)/double(TotBin);
      interdnl=0.;
      for(int ibin=0;ibin<TotBin;ibin++){
        float dnl=(lsbdata[ibin] - expectedhit )/expectedhit;
        interdnl=interdnl+dnl;
        int tempinl=int(interdnl*10000.);
        inlcor[ibin]=tempinl/10000.;
        dnl1024h[itdc][ich]->SetBinContent(ibin,dnl);
        inl1024h[itdc][ich]->SetBinContent(ibin,interdnl);
      }
      // write out INL table
      sprintf(tmpchr,"../result/%s/inl/%s.tdc%d.ch%d.inl",filename,filename,itdc,ich);
      if(processmode=="1")sprintf(tmpchr,"../result/%s/inl/%s.tdc%d.inl",filename,filename,itdc);
      ofstream inltablefile(tmpchr);
      cout<<"Write out inltable(1024) to:"<<tmpchr<<endl;
      int startbin=0;
      for(int ibin=startbin;ibin<startbin+1024;ibin++){
        inltablefile<<setw(5)<<ibin+0.5-startbin<<" "<<setw(10)<<inlcor[ibin]<<endl;
      }
      inltablefile.close();
    }
  }
  // write everything to root file
  sprintf(tmpchr,"../result/%s/root/%s.inl.root",filename,filename);
  cout<<"Write to root file :"<<tmpchr<<endl;
  TFile* fFile=new TFile(tmpchr,"recreate");
  for(int itdc=0;itdc<3;itdc++){
    for(int ich=0;ich<8;ich++){
      lsb2048h[itdc][ich]->Write();
      dnl2048h[itdc][ich]->Write();dnl1024h[itdc][ich]->Write();
      inl2048h[itdc][ich]->Write();inl1024h[itdc][ich]->Write();
    }
  }
  /*
  tdc0ch0time1h->Write();
  tdc1ch0time1h->Write();
  tdc2ch0time1h->Write();
  */
  fFile->Close();

  //----------------------------------------------------
  //--------------- plot the INL plots to gif file.
  // copyt from plot8inl.C
  // 
      
//=====================================================
  gROOT->SetStyle("Plain");

  gStyle->SetCanvasBorderMode(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetPadBorderMode(0);
  gStyle->SetDrawBorder(0);
  gStyle->SetCanvasBorderSize(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetStatBorderSize(1);
  gStyle->SetTitleBorderSize(0);

  // Say it in black and white!
  gStyle->SetLabelFont(62,"xyz");
  //gStyle->SetStatFont(42);
  //gStyle->SetTitleFont(42);
  gStyle->SetTitleOffset(1.25,"Y");
  gStyle->SetTitleOffset(1.,"X");
  gStyle->SetTitleSize(0.06);
  gStyle->SetOptFile(1);

  // Set Line Widths
  gStyle->SetFrameLineWidth(2);
  gStyle->SetFuncWidth(2);
  //gStyle->SetHistLineWidth(2);

  // Set margins -- I like to shift the plot a little up and to the
  // right to make more room for axis labels
  gStyle->SetPadTopMargin(0.08);
  gStyle->SetPadBottomMargin(0.12);
  gStyle->SetPadLeftMargin(0.12);
  gStyle->SetPadRightMargin(0.08);
  //
  // Set Data/Stat/... and other options
  // Set tick marks and turn off grids
  gStyle->SetNdivisions(510,"xyz");
  //gStyle->SetPadTickX(1);
  //gStyle->SetPadTickY(1);
  //gStyle->SetTickLength(0.02,"xyz");
  gStyle->SetPadGridX(0);
  gStyle->SetPadGridY(1);
  // Adjust size and placement of axis labels
  gStyle->SetLabelSize(0.06,"x");
  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetLabelOffset(0.005,"x");
  gStyle->SetLabelOffset(0.01,"y");
  // Set paper size for life in the US
  gStyle->SetPaperSize(TStyle::kUSLetter);

  gStyle->SetPalette(1);
  //
  gStyle->SetOptStat(0);
  gStyle->SetStatFont(30);
  gStyle->SetStatTextColor(4);
  gStyle->SetStatColor(0);
  gStyle->SetStatX(0.95); gStyle->SetStatY(0.92);
  gStyle->SetStatW(0.42); gStyle->SetStatH(0.20);

  gStyle->SetTitleX(0.25);gStyle->SetTitleY(0.985);
  gStyle->SetTitleW(0.7);gStyle->SetTitleH(0.065);
  gStyle->SetTitleColor(0);gStyle->SetTitleTextColor(2);
  gStyle->SetOptTitle(0);

  gROOT->ForceStyle();
  //=============================================

  sprintf(tmpchr,"../result/%s/root/%s.inl.root",filename,filename);
  TFile* hfile=new TFile(tmpchr,"READ");

  TCanvas* c1=new TCanvas("c1","c1",0,0,750,650);
  c1->SetFillColor(0);c1->Divide(1,3,0,0);
  // TGaxis* A=new TGaxis(); A->SetMaxDigits(2);
  //Global title
  //sprintf(tmpchr,"%s INL ",filename);
  //TPaveLabel* gtitle1=new TPaveLabel(0.3,0.96,0.7,0.999,tmpchr);
  //gtitle1->SetFillColor(0);gtitle1->SetTextColor(4);
  //gtitle1->Draw();
  TPad* pad[3];
  for(int itdc=0;itdc<3;itdc++){
    c1->cd(itdc+1);
    pad[itdc]=(TPad*) gPad;
 
    pad[itdc]->SetTopMargin(0.05);
    if(itdc>0)pad[itdc]->SetTopMargin(0);
    pad[itdc]->SetBottomMargin(0.12);
    if(itdc<2)pad[itdc]->SetBottomMargin(0.);
    pad[itdc]->SetTickx();
    //gPad->SetTicky();
    sprintf(tmpchr,"TDC%dch0_inl1024h",itdc);
    TH1F* temph=(TH1F*) hfile->Get(tmpchr);

    float hmax=temph->GetMaximum();
    float hmin=temph->GetMinimum();
    temph->SetMaximum(hmax+10);temph->SetMinimum(hmin-4);
    temph->Draw();
    
    TLegend* leg1=new TLegend(0.12,0.72,0.18,0.95);leg1->SetFillColor(0);leg1->SetBorderSize(1);
    TLegend* leg2=new TLegend(0.18,0.72,0.24,0.95);leg2->SetFillColor(0);leg2->SetBorderSize(1);
    sprintf(tmpchr,"ch%d",0);leg1->AddEntry(temph,tmpchr,"lp");

    TLegend* leg3=new TLegend(0.25,0.745,0.65,0.945);leg3->SetFillColor(0);leg3->SetBorderSize(0);
    sprintf(tmpchr,"%s TDC%d INL",filename,itdc);
    leg3->AddEntry(temph,tmpchr,"");
    
    for(int ich=1;ich<8;ich++){
       sprintf(tmpchr,"TDC%dch%d_inl1024h",itdc,ich);
       temph=(TH1F*) hfile->Get(tmpchr);
       temph->SetLineColor(ich+1);
       sprintf(tmpchr,"ch%d",ich);
       if(ich<4)leg1->AddEntry(temph,tmpchr,"lp");  
       if(ich>=4)leg2->AddEntry(temph,tmpchr,"lp");  
       temph->Draw("same");
    }

    TLatex *   tex = new TLatex(-54.,-3.5,"INL corr (bin)");
    tex->SetTextAngle(90);
    tex->SetTextSize(0.08);
    tex->SetLineWidth(2);
    tex->Draw();
    if(itdc==2) {
      tex = new TLatex(910.707,hmin-7.5,"Bins");
      tex->SetTextSize(0.095);
      tex->SetLineWidth(2);
      tex->Draw();
    }
    if(processmode=="8"){leg1->Draw();leg2->Draw();}
    leg3->Draw();
  }
  //c1->Modified();
  //gtitle1->Draw();
  sprintf(tmpchr,"../result/%s/gif/%s-inl.gif",filename,filename);
  if(processmode=="1")sprintf(tmpchr,"../result/%s/gif/%s-inl1.gif",filename,filename);
  c1->Print(tmpchr); 

  //
  /*
  if (gROOT->IsBatch())  {
    sprintf(tmpchr,"pstopnm -ppm -xborder 0 -yborder 0 -portrait ../%s/gif/%s-inl.eps",filename,filename);
    cout<<tmpchr<<endl;
    gSystem->Exec(tmpchr);
    sprintf(tmpchr,"ppmtogif ../%s/gif/%s-inl.eps001.ppm > ../%s/gif/%s-inl.gif",filename,filename,filename,filename);
    cout<<tmpchr<<endl;
    gSystem->Exec(tmpchr);
    sprintf(tmpchr,"rm -f ../%s/gif/%s-inl.eps001.ppm",filename,filename);
    cout<<tmpchr<<endl;   
    gSystem->Exec(tmpchr);
  }
  */
  // -- 
  // theApp->Run();  
  //=================================================================================
  cout<<" All done. "<<endl;
  // time statistics.
  gBenchmark->Stop("main");       // run time stop here
  gBenchmark->Show("main");       // print out the time information
  float realtime = gBenchmark->GetRealTime("main");        // real time elapsed
  float cputime  = gBenchmark->GetCpuTime("main");         // CPU time we use
  cout<<" real time = "<<realtime<<" seconds cput="<<cputime<<" seconds"<<endl;

  return 0;
}


//-------------------------------
void CreateDir(const string directoryname)
{
  string command;
  // if the directoryname does not exist, creat it
  if(gSystem->AccessPathName(directoryname.c_str())){
    command = "mkdir "+directoryname; 
    cout<<"mkdir :"<<directoryname<<endl;
    gSystem->Exec(command.c_str()); //execute shell command to cread directory
  }
}
