#include <iostream>
#include <string>
#include <fstream>
#include <iomanip>
#include <TFile.h>
#include <TH1.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TPaveLabel.h>
#include <TVirtualPad.h>
#include <TGaxis.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TLegend.h>
#include <TLatex.h>
#include <TROOT.h>
#include <TBenchmark.h>     // ROOT head file for benchmarking applications
#include <TApplication.h>
#include <TH2.h>

#include <TBenchmark.h>     // ROOT head file for benchmarking applications

using namespace std;

int main(int argc, char *argv[])
{

  // need this line to make "gif" files online....do not run code in batch mode.
  TApplication* theApp=new TApplication("app",&argc ,argv);

  // time statistic --- start
  TBenchmark* gBenchmark = new TBenchmark();
  //start time of this program
  gBenchmark->Start("main");

  // process commandline
  if(argc<2) {
    cout<<"Usage: summary_analysis -h or summary_analysis --help to get help"<<endl;
    return -1;
  }
  string datafilename="unknown";
  string processmode="8";
  if (argc>1){
    for (int i=1;i<(argc);i++){
      if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")){
        cout<<"Usage: ----------->"<<endl;
        cout<<argv[0]<<" -mode 1, or 8 "<<endl;
        cout<<argv[0]<<" -f datafilename "<<endl;
        return 0;
      }
      if (!strcmp(argv[i],"-f"))   {datafilename=argv[++i];}
      if (!strcmp(argv[i],"-mode"))   {processmode=argv[++i];}
    }
  }

  char tmpchr[200];
  string ComFileName=datafilename;
  int dotpos=ComFileName.find(".dat");  
  if(dotpos>0)ComFileName=ComFileName.erase(dotpos,ComFileName.length());  
  cout<<"summary_analysis: "<<ComFileName<<endl;
  char filename[200];
  sprintf(filename,"%s",ComFileName.c_str());

//=========================================================

gROOT->SetStyle("Plain");
// Turn off all borders
gStyle->SetCanvasBorderMode(0);
gStyle->SetFrameBorderMode(0);
gStyle->SetPadBorderMode(0);
gStyle->SetDrawBorder(0);
gStyle->SetCanvasBorderSize(0);
gStyle->SetFrameBorderSize(0);
gStyle->SetPadBorderSize(0);
gStyle->SetTitleBorderSize(1);

gStyle->SetNdivisions(505,"xyz");

gStyle->SetLabelSize(0.06,"xyz");
gStyle->SetLabelSize(0.05,"y");
gStyle->SetLabelSize(0.08,"x");
gStyle->SetLabelOffset(0.01,"x");
gStyle->SetLabelOffset(0.01,"y");
gStyle->SetTickLength(0.09);
gStyle->SetTitleColor(2,"xyz");
gStyle->SetTitleOffset(1.3,"y");

gStyle->SetTitleX(0.); gStyle->SetTitleY(1.);
gStyle->SetTitleW(0.98); gStyle->SetTitleH(0.08);
gStyle->SetOptTitle(0);

gStyle->SetStatX(0.90); gStyle->SetStatY(0.90);
gStyle->SetStatW(0.20); gStyle->SetStatH(0.15);

gStyle->SetOptFit(11);
gStyle->SetOptStat(0);

gStyle->SetPadGridX(0);
gStyle->SetPadGridY(0);

//====================================================================
  TFile* hfile[3];
  for(int itdc=0;itdc<3;itdc++){
    sprintf(tmpchr,"../result/%s/root/%s.tdc%d.corr8.root",filename,filename,itdc);
    if(processmode=="1")sprintf(tmpchr,"../result/%s/root/%s.tdc%d.corr1.root",filename,filename,itdc);
    cout<<"Open: "<<tmpchr<<endl;
    hfile[itdc]=new TFile(tmpchr,"READ");
  }
  TH2F* meanvssigma2h[3];
  TH2F* ndtimevssigma2h[3];
  TH2F* avesigmavschan2h[3];
  TH1F* sigma1h[3];
  float maxsigma=2.;
  float minsigma=0.;
  for(int itdc=0;itdc<3;itdc++){
    sprintf(tmpchr,"ndtime_vs_sigma_tdc%d",itdc);
    ndtimevssigma2h[itdc]=new TH2F(tmpchr,tmpchr,280,-0.5,28.5,300,minsigma,maxsigma);
    sprintf(tmpchr,"mean_vs_sigma_tdc%d",itdc);
    meanvssigma2h[itdc]=new TH2F(tmpchr,tmpchr,1000,0.,20000.5,50,0.,3.);
    sprintf(tmpchr,"Avesigma_vs_chan_tdc%d",itdc);
    avesigmavschan2h[itdc]=new TH2F(tmpchr,tmpchr,80,-0.5,7.5,300,minsigma,maxsigma-0.2);

    sprintf(tmpchr,"TDC%d_all_sigma",itdc);
    sigma1h[itdc]=new TH1F(tmpchr,tmpchr,60,0.,3.);
  }
  TH1F* avechansigma1h[3][8];
  for(int itdc=0;itdc<3;itdc++){
    for(int ich=0;ich<8;ich++){
      sprintf(tmpchr,"avesigma_tdc%d_ch%d",itdc,ich);
      avechansigma1h[itdc][ich]=new TH1F(tmpchr,tmpchr,30,0.,3.);
    }
  }
  int icolor[3]={2,3,4};
  int imarker[3]={20,21,23};
  float fmarkersize=1.1;
  for(int itdc=0;itdc<3;itdc++){
    hfile[itdc]->cd();
    for(int i=0;i<8;i++){
      for(int j=0;j<8;j++){
        if(j==i) continue;
        sprintf(tmpchr,"TDC%d_deltaT_ch%d_ch%d",itdc,i,j);
        TH1F* temp0h=(TH1F*)hfile[itdc]->Get(tmpchr);
        float mean0=temp0h->GetMean();
        float sigma0=temp0h->GetRMS(); 
        if(sigma0>3) continue;    // to get rid of some big RMS due to bad/dead channel 
        temp0h->SetAxisRange(mean0-50,mean0+50,"X");
        sigma0=temp0h->GetRMS(); 
        avechansigma1h[itdc][i]->Fill(sigma0);
      }
      avesigmavschan2h[itdc]->Fill(i,avechansigma1h[itdc][i]->GetMean());
      avesigmavschan2h[itdc]->SetMarkerColor(icolor[itdc]);
      avesigmavschan2h[itdc]->SetMarkerStyle(imarker[itdc]);
      avesigmavschan2h[itdc]->SetMarkerSize(fmarkersize);

    }
  }
  //--------
  for(int itdc=0;itdc<3;itdc++){
    int ncomb=0;
    hfile[itdc]->cd();

    for(int i=0;i<8;i++){
      for(int j=0;j<8;j++){
        if(j<=i) continue;
        ncomb++;

        sprintf(tmpchr,"TDC%d_deltaT_ch%d_ch%d",itdc,i,j);
        TH1F* temp0h=(TH1F*)hfile[itdc]->Get(tmpchr);
        float mean0=temp0h->GetMean();
        float sigma0=temp0h->GetRMS(); 
        if(sigma0>3) continue;    // to get rid of some big RMS due to bad/dead channel 
        temp0h->SetAxisRange(mean0-50,mean0+50,"X");
        sigma0=temp0h->GetRMS(); 

        meanvssigma2h[itdc]->Fill(mean0,sigma0);
        sigma1h[itdc]->Fill(sigma0);
        ndtimevssigma2h[itdc]->Fill(ncomb,sigma0);

        ndtimevssigma2h[itdc]->SetMarkerColor(icolor[itdc]);
        ndtimevssigma2h[itdc]->SetMarkerStyle(imarker[itdc]);
        ndtimevssigma2h[itdc]->SetMarkerSize(fmarkersize);
        meanvssigma2h[itdc]->SetMarkerColor(icolor[itdc]);
        meanvssigma2h[itdc]->SetMarkerStyle(imarker[itdc]);
        meanvssigma2h[itdc]->SetMarkerSize(fmarkersize);
        sigma1h[itdc]->SetLineColor(icolor[itdc]);
        sigma1h[itdc]->SetLineWidth(2);
      }
    }
  }
  // write everything to root file

  sprintf(tmpchr,"../result/%s/root/%s.summary.root",ComFileName.c_str(),ComFileName.c_str());
  if(processmode=="1")sprintf(tmpchr,"../result/%s/root/%s.summary1.root",ComFileName.c_str(),ComFileName.c_str());
  cout<<"Write to root file :"<<tmpchr<<endl;
  TFile* fFile=new TFile(tmpchr,"recreate");
  for(int itdc=0;itdc<3;itdc++){
    ndtimevssigma2h[itdc]->Write();
    meanvssigma2h[itdc]->Write(); 
    avesigmavschan2h[itdc]->Write();
    sigma1h[itdc]->Write();
  }

  fFile->Close();


  //===============================================================================
  TLatex* tex;
  TCanvas *c1 = new TCanvas("c1", "c1",0,0,750,650);
  c1->Divide(1,2,0.01,0.01);c1->SetFillColor(0);

  c1->cd(1);
  gPad->SetGridy(1);
  TLegend *leg1 = new TLegend(0.11,0.70,0.23,0.90);leg1->SetBorderSize(1);leg1->SetFillColor(0);
  leg1->AddEntry(ndtimevssigma2h[0],"TDC0","lp");
  leg1->AddEntry(ndtimevssigma2h[1],"TDC1","lp");
  leg1->AddEntry(ndtimevssigma2h[2],"TDC2","lp");

  ndtimevssigma2h[0]->Draw();
  ndtimevssigma2h[1]->Draw("same");
  ndtimevssigma2h[2]->Draw("same");
  int ncomb=0;
  for(int i=0;i<8;i++){
    for(int j=0;j<8;j++){
      if(j<=i) continue;
      ncomb++;

      sprintf(tmpchr,"ch %d-%d",i,j);
      float xpos=ncomb-0.44;
      tex = new TLatex(xpos,minsigma,tmpchr);
      tex->SetTextColor(ncomb%2+7);
      tex->SetTextSize(0.044);tex->SetTextAngle(70);tex->SetLineWidth(1);
      tex->Draw();
    }
  }
 
  tex = new TLatex(-2.1,1.0,"Sigma (bin)");
  tex->SetTextAngle(90);tex->SetLineWidth(2);
  tex->Draw();
  tex = new TLatex(22,minsigma-0.2,"N (ch x-ch y)");
  tex->SetTextSize(0.08);tex->SetLineWidth(2);
  tex->Draw();
  //----
  sprintf(tmpchr,"%s",filename);
  tex = new TLatex(8.9,maxsigma-0.3,tmpchr);
  tex->SetLineWidth(2);
  tex->SetTextSize(0.085);
  tex->Draw();

  leg1->Draw();
  //=======================================================================
  c1->cd(2);

  gPad->SetGridy(1);
  TLegend *leg2 = new TLegend(0.11,0.70,0.23,0.90);leg2->SetBorderSize(1);leg2->SetFillColor(0);
  leg2->AddEntry(avesigmavschan2h[0],"TDC0","lp");
  leg2->AddEntry(avesigmavschan2h[1],"TDC1","lp");
  leg2->AddEntry(avesigmavschan2h[2],"TDC2","lp");

  avesigmavschan2h[0]->Draw();
  avesigmavschan2h[1]->Draw("same");
  avesigmavschan2h[2]->Draw("same");
 
  tex = new TLatex(-0.9,0.85,"Ave. Sigma (bin)");
  tex->SetTextAngle(90);tex->SetLineWidth(2);
  tex->Draw();
  tex = new TLatex(6.5,minsigma-0.20,"Chan #");
  tex->SetTextSize(0.075);tex->SetLineWidth(2);
  tex->Draw();

  leg2->Draw();

  //============================================================================
  sprintf(tmpchr,"../result/%s/gif/%s-summary.gif",filename,filename);
  if(processmode=="1")sprintf(tmpchr,"../result/%s/gif/%s-summary1.gif",filename,filename);
  c1->Print(tmpchr);
  //
  // end plot

  //----------------------------------------------------
  cout<<" All done. "<<endl;
  // time statistics.
  gBenchmark->Stop("main");       // run time stop here
  gBenchmark->Show("main");       // print out the time information
  float realtime = gBenchmark->GetRealTime("main");        // real time elapsed
  float cputime  = gBenchmark->GetCpuTime("main");         // CPU time we use
  cout<<" real time = "<<realtime<<" seconds cput="<<cputime<<" seconds"<<endl;

  return 0;
}

